# Generated from Spud.g4 by ANTLR 4.9.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\36")
        buf.write("\u025a\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\4\64\t\64")
        buf.write("\4\65\t\65\4\66\t\66\4\67\t\67\48\t8\49\t9\4:\t:\4;\t")
        buf.write(";\3\2\7\2x\n\2\f\2\16\2{\13\2\3\2\3\2\7\2\177\n\2\f\2")
        buf.write("\16\2\u0082\13\2\3\3\3\3\3\3\7\3\u0087\n\3\f\3\16\3\u008a")
        buf.write("\13\3\3\4\7\4\u008d\n\4\f\4\16\4\u0090\13\4\3\4\3\4\3")
        buf.write("\4\3\5\3\5\7\5\u0097\n\5\f\5\16\5\u009a\13\5\3\5\3\5\7")
        buf.write("\5\u009e\n\5\f\5\16\5\u00a1\13\5\3\5\3\5\3\6\3\6\6\6\u00a7")
        buf.write("\n\6\r\6\16\6\u00a8\3\6\3\6\3\7\3\7\3\7\3\7\3\7\5\7\u00b2")
        buf.write("\n\7\3\b\3\b\7\b\u00b6\n\b\f\b\16\b\u00b9\13\b\3\t\3\t")
        buf.write("\7\t\u00bd\n\t\f\t\16\t\u00c0\13\t\3\t\3\t\3\n\3\n\5\n")
        buf.write("\u00c6\n\n\3\13\7\13\u00c9\n\13\f\13\16\13\u00cc\13\13")
        buf.write("\3\13\3\13\7\13\u00d0\n\13\f\13\16\13\u00d3\13\13\3\13")
        buf.write("\3\13\3\f\7\f\u00d8\n\f\f\f\16\f\u00db\13\f\3\r\7\r\u00de")
        buf.write("\n\r\f\r\16\r\u00e1\13\r\3\r\3\r\7\r\u00e5\n\r\f\r\16")
        buf.write("\r\u00e8\13\r\3\r\3\r\3\16\7\16\u00ed\n\16\f\16\16\16")
        buf.write("\u00f0\13\16\3\17\7\17\u00f3\n\17\f\17\16\17\u00f6\13")
        buf.write("\17\3\17\3\17\7\17\u00fa\n\17\f\17\16\17\u00fd\13\17\3")
        buf.write("\17\3\17\3\20\3\20\7\20\u0103\n\20\f\20\16\20\u0106\13")
        buf.write("\20\3\21\3\21\3\21\7\21\u010b\n\21\f\21\16\21\u010e\13")
        buf.write("\21\3\22\3\22\3\22\3\23\3\23\6\23\u0115\n\23\r\23\16\23")
        buf.write("\u0116\3\23\3\23\6\23\u011b\n\23\r\23\16\23\u011c\5\23")
        buf.write("\u011f\n\23\5\23\u0121\n\23\3\23\3\23\3\24\5\24\u0126")
        buf.write("\n\24\3\24\3\24\5\24\u012a\n\24\3\25\3\25\3\25\3\26\3")
        buf.write("\26\3\26\3\27\3\27\3\27\3\30\3\30\3\30\3\31\3\31\5\31")
        buf.write("\u013a\n\31\3\32\3\32\5\32\u013e\n\32\3\33\3\33\3\33\7")
        buf.write("\33\u0143\n\33\f\33\16\33\u0146\13\33\3\34\3\34\3\34\7")
        buf.write("\34\u014b\n\34\f\34\16\34\u014e\13\34\3\35\3\35\3\35\7")
        buf.write("\35\u0153\n\35\f\35\16\35\u0156\13\35\3\36\3\36\3\36\3")
        buf.write("\36\3\37\3\37\3\37\3\37\3 \3 \3 \3 \3!\3!\6!\u0166\n!")
        buf.write("\r!\16!\u0167\3!\3!\3\"\3\"\5\"\u016e\n\"\3\"\6\"\u0171")
        buf.write("\n\"\r\"\16\"\u0172\3\"\3\"\3#\3#\3#\7#\u017a\n#\f#\16")
        buf.write("#\u017d\13#\3$\3$\3$\7$\u0182\n$\f$\16$\u0185\13$\3%\3")
        buf.write("%\3%\3%\3&\3&\6&\u018d\n&\r&\16&\u018e\3&\3&\3\'\3\'\6")
        buf.write("\'\u0195\n\'\r\'\16\'\u0196\3\'\3\'\3(\3(\6(\u019d\n(")
        buf.write("\r(\16(\u019e\3(\3(\5(\u01a3\n(\3)\3)\5)\u01a7\n)\3)\6")
        buf.write(")\u01aa\n)\r)\16)\u01ab\3)\3)\5)\u01b0\n)\3)\6)\u01b3")
        buf.write("\n)\r)\16)\u01b4\3)\3)\3*\3*\3*\7*\u01bc\n*\f*\16*\u01bf")
        buf.write("\13*\3+\3+\3+\7+\u01c4\n+\f+\16+\u01c7\13+\3,\3,\3,\3")
        buf.write(",\3-\3-\3-\3-\3-\5-\u01d2\n-\3.\3.\6.\u01d6\n.\r.\16.")
        buf.write("\u01d7\3.\7.\u01db\n.\f.\16.\u01de\13.\3/\3/\6/\u01e2")
        buf.write("\n/\r/\16/\u01e3\3/\3/\3\60\3\60\6\60\u01ea\n\60\r\60")
        buf.write("\16\60\u01eb\3\60\3\60\5\60\u01f0\n\60\3\61\6\61\u01f3")
        buf.write("\n\61\r\61\16\61\u01f4\3\61\3\61\5\61\u01f9\n\61\3\62")
        buf.write("\3\62\3\62\3\63\3\63\3\63\3\64\3\64\3\64\3\65\3\65\3\65")
        buf.write("\3\66\3\66\6\66\u0209\n\66\r\66\16\66\u020a\3\66\3\66")
        buf.write("\6\66\u020f\n\66\r\66\16\66\u0210\3\66\3\66\7\66\u0215")
        buf.write("\n\66\f\66\16\66\u0218\13\66\3\66\5\66\u021b\n\66\3\67")
        buf.write("\7\67\u021e\n\67\f\67\16\67\u0221\13\67\3\67\3\67\38\3")
        buf.write("8\38\38\38\78\u022a\n8\f8\168\u022d\138\38\38\39\39\3")
        buf.write("9\39\39\39\59\u0237\n9\3:\7:\u023a\n:\f:\16:\u023d\13")
        buf.write(":\3:\3:\3:\5:\u0242\n:\3:\7:\u0245\n:\f:\16:\u0248\13")
        buf.write(":\5:\u024a\n:\3:\5:\u024d\n:\3;\3;\3;\7;\u0252\n;\f;\16")
        buf.write(";\u0255\13;\3;\3;\3;\3;\2\2<\2\4\6\b\n\f\16\20\22\24\26")
        buf.write("\30\32\34\36 \"$&(*,.\60\62\64\668:<>@BDFHJLNPRTVXZ\\")
        buf.write("^`bdfhjlnprt\2\7\3\2\27\30\3\2\5\6\3\2\26\26\3\2\b\b\3")
        buf.write("\2\t\n\2\u026f\2y\3\2\2\2\4\u0083\3\2\2\2\6\u008e\3\2")
        buf.write("\2\2\b\u0094\3\2\2\2\n\u00a4\3\2\2\2\f\u00b1\3\2\2\2\16")
        buf.write("\u00b3\3\2\2\2\20\u00ba\3\2\2\2\22\u00c5\3\2\2\2\24\u00ca")
        buf.write("\3\2\2\2\26\u00d9\3\2\2\2\30\u00df\3\2\2\2\32\u00ee\3")
        buf.write("\2\2\2\34\u00f4\3\2\2\2\36\u0104\3\2\2\2 \u0107\3\2\2")
        buf.write("\2\"\u010f\3\2\2\2$\u0120\3\2\2\2&\u0125\3\2\2\2(\u012b")
        buf.write("\3\2\2\2*\u012e\3\2\2\2,\u0131\3\2\2\2.\u0134\3\2\2\2")
        buf.write("\60\u0139\3\2\2\2\62\u013d\3\2\2\2\64\u013f\3\2\2\2\66")
        buf.write("\u0147\3\2\2\28\u014f\3\2\2\2:\u0157\3\2\2\2<\u015b\3")
        buf.write("\2\2\2>\u015f\3\2\2\2@\u0163\3\2\2\2B\u016d\3\2\2\2D\u0176")
        buf.write("\3\2\2\2F\u017e\3\2\2\2H\u0186\3\2\2\2J\u018a\3\2\2\2")
        buf.write("L\u0192\3\2\2\2N\u019a\3\2\2\2P\u01a6\3\2\2\2R\u01b8\3")
        buf.write("\2\2\2T\u01c0\3\2\2\2V\u01c8\3\2\2\2X\u01d1\3\2\2\2Z\u01d3")
        buf.write("\3\2\2\2\\\u01df\3\2\2\2^\u01e7\3\2\2\2`\u01f2\3\2\2\2")
        buf.write("b\u01fa\3\2\2\2d\u01fd\3\2\2\2f\u0200\3\2\2\2h\u0203\3")
        buf.write("\2\2\2j\u0206\3\2\2\2l\u021f\3\2\2\2n\u0224\3\2\2\2p\u0236")
        buf.write("\3\2\2\2r\u023b\3\2\2\2t\u0253\3\2\2\2vx\7\27\2\2wv\3")
        buf.write("\2\2\2x{\3\2\2\2yw\3\2\2\2yz\3\2\2\2z|\3\2\2\2{y\3\2\2")
        buf.write("\2|\u0080\7\30\2\2}\177\t\2\2\2~}\3\2\2\2\177\u0082\3")
        buf.write("\2\2\2\u0080~\3\2\2\2\u0080\u0081\3\2\2\2\u0081\3\3\2")
        buf.write("\2\2\u0082\u0080\3\2\2\2\u0083\u0084\5\2\2\2\u0084\u0088")
        buf.write("\7\3\2\2\u0085\u0087\t\2\2\2\u0086\u0085\3\2\2\2\u0087")
        buf.write("\u008a\3\2\2\2\u0088\u0086\3\2\2\2\u0088\u0089\3\2\2\2")
        buf.write("\u0089\5\3\2\2\2\u008a\u0088\3\2\2\2\u008b\u008d\7\27")
        buf.write("\2\2\u008c\u008b\3\2\2\2\u008d\u0090\3\2\2\2\u008e\u008c")
        buf.write("\3\2\2\2\u008e\u008f\3\2\2\2\u008f\u0091\3\2\2\2\u0090")
        buf.write("\u008e\3\2\2\2\u0091\u0092\7\3\2\2\u0092\u0093\5\2\2\2")
        buf.write("\u0093\7\3\2\2\2\u0094\u0098\5\2\2\2\u0095\u0097\7\25")
        buf.write("\2\2\u0096\u0095\3\2\2\2\u0097\u009a\3\2\2\2\u0098\u0096")
        buf.write("\3\2\2\2\u0098\u0099\3\2\2\2\u0099\u009b\3\2\2\2\u009a")
        buf.write("\u0098\3\2\2\2\u009b\u009f\7\4\2\2\u009c\u009e\7\25\2")
        buf.write("\2\u009d\u009c\3\2\2\2\u009e\u00a1\3\2\2\2\u009f\u009d")
        buf.write("\3\2\2\2\u009f\u00a0\3\2\2\2\u00a0\u00a2\3\2\2\2\u00a1")
        buf.write("\u009f\3\2\2\2\u00a2\u00a3\5\2\2\2\u00a3\t\3\2\2\2\u00a4")
        buf.write("\u00a6\5\2\2\2\u00a5\u00a7\7\25\2\2\u00a6\u00a5\3\2\2")
        buf.write("\2\u00a7\u00a8\3\2\2\2\u00a8\u00a6\3\2\2\2\u00a8\u00a9")
        buf.write("\3\2\2\2\u00a9\u00aa\3\2\2\2\u00aa\u00ab\5\b\5\2\u00ab")
        buf.write("\13\3\2\2\2\u00ac\u00b2\5\2\2\2\u00ad\u00b2\5\4\3\2\u00ae")
        buf.write("\u00b2\5\6\4\2\u00af\u00b2\5\b\5\2\u00b0\u00b2\5\n\6\2")
        buf.write("\u00b1\u00ac\3\2\2\2\u00b1\u00ad\3\2\2\2\u00b1\u00ae\3")
        buf.write("\2\2\2\u00b1\u00af\3\2\2\2\u00b1\u00b0\3\2\2\2\u00b2\r")
        buf.write("\3\2\2\2\u00b3\u00b7\t\3\2\2\u00b4\u00b6\n\4\2\2\u00b5")
        buf.write("\u00b4\3\2\2\2\u00b6\u00b9\3\2\2\2\u00b7\u00b5\3\2\2\2")
        buf.write("\u00b7\u00b8\3\2\2\2\u00b8\17\3\2\2\2\u00b9\u00b7\3\2")
        buf.write("\2\2\u00ba\u00be\7\7\2\2\u00bb\u00bd\n\5\2\2\u00bc\u00bb")
        buf.write("\3\2\2\2\u00bd\u00c0\3\2\2\2\u00be\u00bc\3\2\2\2\u00be")
        buf.write("\u00bf\3\2\2\2\u00bf\u00c1\3\2\2\2\u00c0\u00be\3\2\2\2")
        buf.write("\u00c1\u00c2\7\b\2\2\u00c2\21\3\2\2\2\u00c3\u00c6\7\26")
        buf.write("\2\2\u00c4\u00c6\5\20\t\2\u00c5\u00c3\3\2\2\2\u00c5\u00c4")
        buf.write("\3\2\2\2\u00c6\23\3\2\2\2\u00c7\u00c9\7\25\2\2\u00c8\u00c7")
        buf.write("\3\2\2\2\u00c9\u00cc\3\2\2\2\u00ca\u00c8\3\2\2\2\u00ca")
        buf.write("\u00cb\3\2\2\2\u00cb\u00cd\3\2\2\2\u00cc\u00ca\3\2\2\2")
        buf.write("\u00cd\u00d1\7\t\2\2\u00ce\u00d0\7\25\2\2\u00cf\u00ce")
        buf.write("\3\2\2\2\u00d0\u00d3\3\2\2\2\u00d1\u00cf\3\2\2\2\u00d1")
        buf.write("\u00d2\3\2\2\2\u00d2\u00d4\3\2\2\2\u00d3\u00d1\3\2\2\2")
        buf.write("\u00d4\u00d5\7\31\2\2\u00d5\25\3\2\2\2\u00d6\u00d8\5\24")
        buf.write("\13\2\u00d7\u00d6\3\2\2\2\u00d8\u00db\3\2\2\2\u00d9\u00d7")
        buf.write("\3\2\2\2\u00d9\u00da\3\2\2\2\u00da\27\3\2\2\2\u00db\u00d9")
        buf.write("\3\2\2\2\u00dc\u00de\7\25\2\2\u00dd\u00dc\3\2\2\2\u00de")
        buf.write("\u00e1\3\2\2\2\u00df\u00dd\3\2\2\2\u00df\u00e0\3\2\2\2")
        buf.write("\u00e0\u00e2\3\2\2\2\u00e1\u00df\3\2\2\2\u00e2\u00e6\t")
        buf.write("\6\2\2\u00e3\u00e5\7\25\2\2\u00e4\u00e3\3\2\2\2\u00e5")
        buf.write("\u00e8\3\2\2\2\u00e6\u00e4\3\2\2\2\u00e6\u00e7\3\2\2\2")
        buf.write("\u00e7\u00e9\3\2\2\2\u00e8\u00e6\3\2\2\2\u00e9\u00ea\7")
        buf.write("\31\2\2\u00ea\31\3\2\2\2\u00eb\u00ed\5\30\r\2\u00ec\u00eb")
        buf.write("\3\2\2\2\u00ed\u00f0\3\2\2\2\u00ee\u00ec\3\2\2\2\u00ee")
        buf.write("\u00ef\3\2\2\2\u00ef\33\3\2\2\2\u00f0\u00ee\3\2\2\2\u00f1")
        buf.write("\u00f3\7\25\2\2\u00f2\u00f1\3\2\2\2\u00f3\u00f6\3\2\2")
        buf.write("\2\u00f4\u00f2\3\2\2\2\u00f4\u00f5\3\2\2\2\u00f5\u00f7")
        buf.write("\3\2\2\2\u00f6\u00f4\3\2\2\2\u00f7\u00fb\7\n\2\2\u00f8")
        buf.write("\u00fa\7\25\2\2\u00f9\u00f8\3\2\2\2\u00fa\u00fd\3\2\2")
        buf.write("\2\u00fb\u00f9\3\2\2\2\u00fb\u00fc\3\2\2\2\u00fc\u00fe")
        buf.write("\3\2\2\2\u00fd\u00fb\3\2\2\2\u00fe\u00ff\7\13\2\2\u00ff")
        buf.write("\35\3\2\2\2\u0100\u0103\5\30\r\2\u0101\u0103\5\34\17\2")
        buf.write("\u0102\u0100\3\2\2\2\u0102\u0101\3\2\2\2\u0103\u0106\3")
        buf.write("\2\2\2\u0104\u0102\3\2\2\2\u0104\u0105\3\2\2\2\u0105\37")
        buf.write("\3\2\2\2\u0106\u0104\3\2\2\2\u0107\u010c\7\31\2\2\u0108")
        buf.write("\u0109\7\3\2\2\u0109\u010b\7\31\2\2\u010a\u0108\3\2\2")
        buf.write("\2\u010b\u010e\3\2\2\2\u010c\u010a\3\2\2\2\u010c\u010d")
        buf.write("\3\2\2\2\u010d!\3\2\2\2\u010e\u010c\3\2\2\2\u010f\u0110")
        buf.write("\5 \21\2\u0110\u0111\5\26\f\2\u0111#\3\2\2\2\u0112\u0114")
        buf.write("\5\f\7\2\u0113\u0115\7\25\2\2\u0114\u0113\3\2\2\2\u0115")
        buf.write("\u0116\3\2\2\2\u0116\u0114\3\2\2\2\u0116\u0117\3\2\2\2")
        buf.write("\u0117\u011e\3\2\2\2\u0118\u011a\5 \21\2\u0119\u011b\7")
        buf.write("\25\2\2\u011a\u0119\3\2\2\2\u011b\u011c\3\2\2\2\u011c")
        buf.write("\u011a\3\2\2\2\u011c\u011d\3\2\2\2\u011d\u011f\3\2\2\2")
        buf.write("\u011e\u0118\3\2\2\2\u011e\u011f\3\2\2\2\u011f\u0121\3")
        buf.write("\2\2\2\u0120\u0112\3\2\2\2\u0120\u0121\3\2\2\2\u0121\u0122")
        buf.write("\3\2\2\2\u0122\u0123\5\"\22\2\u0123%\3\2\2\2\u0124\u0126")
        buf.write("\7\f\2\2\u0125\u0124\3\2\2\2\u0125\u0126\3\2\2\2\u0126")
        buf.write("\u0127\3\2\2\2\u0127\u0129\5 \21\2\u0128\u012a\7\f\2\2")
        buf.write("\u0129\u0128\3\2\2\2\u0129\u012a\3\2\2\2\u012a\'\3\2\2")
        buf.write("\2\u012b\u012c\5&\24\2\u012c\u012d\5\32\16\2\u012d)\3")
        buf.write("\2\2\2\u012e\u012f\5 \21\2\u012f\u0130\5\36\20\2\u0130")
        buf.write("+\3\2\2\2\u0131\u0132\7\13\2\2\u0132\u0133\5\32\16\2\u0133")
        buf.write("-\3\2\2\2\u0134\u0135\7\13\2\2\u0135\u0136\5\36\20\2\u0136")
        buf.write("/\3\2\2\2\u0137\u013a\5(\25\2\u0138\u013a\5,\27\2\u0139")
        buf.write("\u0137\3\2\2\2\u0139\u0138\3\2\2\2\u013a\61\3\2\2\2\u013b")
        buf.write("\u013e\5*\26\2\u013c\u013e\5.\30\2\u013d\u013b\3\2\2\2")
        buf.write("\u013d\u013c\3\2\2\2\u013e\63\3\2\2\2\u013f\u0144\5(\25")
        buf.write("\2\u0140\u0141\7\32\2\2\u0141\u0143\5(\25\2\u0142\u0140")
        buf.write("\3\2\2\2\u0143\u0146\3\2\2\2\u0144\u0142\3\2\2\2\u0144")
        buf.write("\u0145\3\2\2\2\u0145\65\3\2\2\2\u0146\u0144\3\2\2\2\u0147")
        buf.write("\u014c\5*\26\2\u0148\u0149\7\32\2\2\u0149\u014b\5*\26")
        buf.write("\2\u014a\u0148\3\2\2\2\u014b\u014e\3\2\2\2\u014c\u014a")
        buf.write("\3\2\2\2\u014c\u014d\3\2\2\2\u014d\67\3\2\2\2\u014e\u014c")
        buf.write("\3\2\2\2\u014f\u0154\5\62\32\2\u0150\u0151\7\32\2\2\u0151")
        buf.write("\u0153\5\62\32\2\u0152\u0150\3\2\2\2\u0153\u0156\3\2\2")
        buf.write("\2\u0154\u0152\3\2\2\2\u0154\u0155\3\2\2\2\u01559\3\2")
        buf.write("\2\2\u0156\u0154\3\2\2\2\u0157\u0158\5\64\33\2\u0158\u0159")
        buf.write("\7\33\2\2\u0159\u015a\58\35\2\u015a;\3\2\2\2\u015b\u015c")
        buf.write("\5\64\33\2\u015c\u015d\7\34\2\2\u015d\u015e\58\35\2\u015e")
        buf.write("=\3\2\2\2\u015f\u0160\5\64\33\2\u0160\u0161\7\35\2\2\u0161")
        buf.write("\u0162\5\66\34\2\u0162?\3\2\2\2\u0163\u0165\5\f\7\2\u0164")
        buf.write("\u0166\7\25\2\2\u0165\u0164\3\2\2\2\u0166\u0167\3\2\2")
        buf.write("\2\u0167\u0165\3\2\2\2\u0167\u0168\3\2\2\2\u0168\u0169")
        buf.write("\3\2\2\2\u0169\u016a\5(\25\2\u016aA\3\2\2\2\u016b\u016e")
        buf.write("\5\f\7\2\u016c\u016e\7\13\2\2\u016d\u016b\3\2\2\2\u016d")
        buf.write("\u016c\3\2\2\2\u016e\u0170\3\2\2\2\u016f\u0171\7\25\2")
        buf.write("\2\u0170\u016f\3\2\2\2\u0171\u0172\3\2\2\2\u0172\u0170")
        buf.write("\3\2\2\2\u0172\u0173\3\2\2\2\u0173\u0174\3\2\2\2\u0174")
        buf.write("\u0175\5\62\32\2\u0175C\3\2\2\2\u0176\u017b\5@!\2\u0177")
        buf.write("\u0178\7\32\2\2\u0178\u017a\5@!\2\u0179\u0177\3\2\2\2")
        buf.write("\u017a\u017d\3\2\2\2\u017b\u0179\3\2\2\2\u017b\u017c\3")
        buf.write("\2\2\2\u017cE\3\2\2\2\u017d\u017b\3\2\2\2\u017e\u0183")
        buf.write("\5B\"\2\u017f\u0180\7\32\2\2\u0180\u0182\5B\"\2\u0181")
        buf.write("\u017f\3\2\2\2\u0182\u0185\3\2\2\2\u0183\u0181\3\2\2\2")
        buf.write("\u0183\u0184\3\2\2\2\u0184G\3\2\2\2\u0185\u0183\3\2\2")
        buf.write("\2\u0186\u0187\5D#\2\u0187\u0188\7\33\2\2\u0188\u0189")
        buf.write("\5F$\2\u0189I\3\2\2\2\u018a\u018c\7\13\2\2\u018b\u018d")
        buf.write("\7\25\2\2\u018c\u018b\3\2\2\2\u018d\u018e\3\2\2\2\u018e")
        buf.write("\u018c\3\2\2\2\u018e\u018f\3\2\2\2\u018f\u0190\3\2\2\2")
        buf.write("\u0190\u0191\5(\25\2\u0191K\3\2\2\2\u0192\u0194\5&\24")
        buf.write("\2\u0193\u0195\7\25\2\2\u0194\u0193\3\2\2\2\u0195\u0196")
        buf.write("\3\2\2\2\u0196\u0194\3\2\2\2\u0196\u0197\3\2\2\2\u0197")
        buf.write("\u0198\3\2\2\2\u0198\u0199\5\60\31\2\u0199M\3\2\2\2\u019a")
        buf.write("\u019c\5\f\7\2\u019b\u019d\7\25\2\2\u019c\u019b\3\2\2")
        buf.write("\2\u019d\u019e\3\2\2\2\u019e\u019c\3\2\2\2\u019e\u019f")
        buf.write("\3\2\2\2\u019f\u01a2\3\2\2\2\u01a0\u01a3\5J&\2\u01a1\u01a3")
        buf.write("\5L\'\2\u01a2\u01a0\3\2\2\2\u01a2\u01a1\3\2\2\2\u01a3")
        buf.write("O\3\2\2\2\u01a4\u01a7\5\f\7\2\u01a5\u01a7\7\13\2\2\u01a6")
        buf.write("\u01a4\3\2\2\2\u01a6\u01a5\3\2\2\2\u01a7\u01a9\3\2\2\2")
        buf.write("\u01a8\u01aa\7\25\2\2\u01a9\u01a8\3\2\2\2\u01aa\u01ab")
        buf.write("\3\2\2\2\u01ab\u01a9\3\2\2\2\u01ab\u01ac\3\2\2\2\u01ac")
        buf.write("\u01af\3\2\2\2\u01ad\u01b0\5 \21\2\u01ae\u01b0\7\13\2")
        buf.write("\2\u01af\u01ad\3\2\2\2\u01af\u01ae\3\2\2\2\u01b0\u01b2")
        buf.write("\3\2\2\2\u01b1\u01b3\7\25\2\2\u01b2\u01b1\3\2\2\2\u01b3")
        buf.write("\u01b4\3\2\2\2\u01b4\u01b2\3\2\2\2\u01b4\u01b5\3\2\2\2")
        buf.write("\u01b5\u01b6\3\2\2\2\u01b6\u01b7\5\62\32\2\u01b7Q\3\2")
        buf.write("\2\2\u01b8\u01bd\5N(\2\u01b9\u01ba\7\32\2\2\u01ba\u01bc")
        buf.write("\5N(\2\u01bb\u01b9\3\2\2\2\u01bc\u01bf\3\2\2\2\u01bd\u01bb")
        buf.write("\3\2\2\2\u01bd\u01be\3\2\2\2\u01beS\3\2\2\2\u01bf\u01bd")
        buf.write("\3\2\2\2\u01c0\u01c5\5P)\2\u01c1\u01c2\7\32\2\2\u01c2")
        buf.write("\u01c4\5P)\2\u01c3\u01c1\3\2\2\2\u01c4\u01c7\3\2\2\2\u01c5")
        buf.write("\u01c3\3\2\2\2\u01c5\u01c6\3\2\2\2\u01c6U\3\2\2\2\u01c7")
        buf.write("\u01c5\3\2\2\2\u01c8\u01c9\5R*\2\u01c9\u01ca\7\33\2\2")
        buf.write("\u01ca\u01cb\5T+\2\u01cbW\3\2\2\2\u01cc\u01d2\5:\36\2")
        buf.write("\u01cd\u01d2\5<\37\2\u01ce\u01d2\5> \2\u01cf\u01d2\5H")
        buf.write("%\2\u01d0\u01d2\5V,\2\u01d1\u01cc\3\2\2\2\u01d1\u01cd")
        buf.write("\3\2\2\2\u01d1\u01ce\3\2\2\2\u01d1\u01cf\3\2\2\2\u01d1")
        buf.write("\u01d0\3\2\2\2\u01d2Y\3\2\2\2\u01d3\u01d5\7\r\2\2\u01d4")
        buf.write("\u01d6\7\25\2\2\u01d5\u01d4\3\2\2\2\u01d6\u01d7\3\2\2")
        buf.write("\2\u01d7\u01d5\3\2\2\2\u01d7\u01d8\3\2\2\2\u01d8\u01dc")
        buf.write("\3\2\2\2\u01d9\u01db\n\4\2\2\u01da\u01d9\3\2\2\2\u01db")
        buf.write("\u01de\3\2\2\2\u01dc\u01da\3\2\2\2\u01dc\u01dd\3\2\2\2")
        buf.write("\u01dd[\3\2\2\2\u01de\u01dc\3\2\2\2\u01df\u01e1\7\13\2")
        buf.write("\2\u01e0\u01e2\7\25\2\2\u01e1\u01e0\3\2\2\2\u01e2\u01e3")
        buf.write("\3\2\2\2\u01e3\u01e1\3\2\2\2\u01e3\u01e4\3\2\2\2\u01e4")
        buf.write("\u01e5\3\2\2\2\u01e5\u01e6\5(\25\2\u01e6]\3\2\2\2\u01e7")
        buf.write("\u01e9\5&\24\2\u01e8\u01ea\7\25\2\2\u01e9\u01e8\3\2\2")
        buf.write("\2\u01ea\u01eb\3\2\2\2\u01eb\u01e9\3\2\2\2\u01eb\u01ec")
        buf.write("\3\2\2\2\u01ec\u01ef\3\2\2\2\u01ed\u01f0\5\60\31\2\u01ee")
        buf.write("\u01f0\7\13\2\2\u01ef\u01ed\3\2\2\2\u01ef\u01ee\3\2\2")
        buf.write("\2\u01f0_\3\2\2\2\u01f1\u01f3\7\25\2\2\u01f2\u01f1\3\2")
        buf.write("\2\2\u01f3\u01f4\3\2\2\2\u01f4\u01f2\3\2\2\2\u01f4\u01f5")
        buf.write("\3\2\2\2\u01f5\u01f8\3\2\2\2\u01f6\u01f9\5\\/\2\u01f7")
        buf.write("\u01f9\5^\60\2\u01f8\u01f6\3\2\2\2\u01f8\u01f7\3\2\2\2")
        buf.write("\u01f9a\3\2\2\2\u01fa\u01fb\7\16\2\2\u01fb\u01fc\5`\61")
        buf.write("\2\u01fcc\3\2\2\2\u01fd\u01fe\7\17\2\2\u01fe\u01ff\5`")
        buf.write("\61\2\u01ffe\3\2\2\2\u0200\u0201\7\20\2\2\u0201\u0202")
        buf.write("\5`\61\2\u0202g\3\2\2\2\u0203\u0204\7\21\2\2\u0204\u0205")
        buf.write("\5`\61\2\u0205i\3\2\2\2\u0206\u0208\7\22\2\2\u0207\u0209")
        buf.write("\7\25\2\2\u0208\u0207\3\2\2\2\u0209\u020a\3\2\2\2\u020a")
        buf.write("\u0208\3\2\2\2\u020a\u020b\3\2\2\2\u020b\u020c\3\2\2\2")
        buf.write("\u020c\u020e\7\23\2\2\u020d\u020f\7\25\2\2\u020e\u020d")
        buf.write("\3\2\2\2\u020f\u0210\3\2\2\2\u0210\u020e\3\2\2\2\u0210")
        buf.write("\u0211\3\2\2\2\u0211\u0212\3\2\2\2\u0212\u0216\5\f\7\2")
        buf.write("\u0213\u0215\7\25\2\2\u0214\u0213\3\2\2\2\u0215\u0218")
        buf.write("\3\2\2\2\u0216\u0214\3\2\2\2\u0216\u0217\3\2\2\2\u0217")
        buf.write("\u021a\3\2\2\2\u0218\u0216\3\2\2\2\u0219\u021b\5\16\b")
        buf.write("\2\u021a\u0219\3\2\2\2\u021a\u021b\3\2\2\2\u021bk\3\2")
        buf.write("\2\2\u021c\u021e\7\25\2\2\u021d\u021c\3\2\2\2\u021e\u0221")
        buf.write("\3\2\2\2\u021f\u021d\3\2\2\2\u021f\u0220\3\2\2\2\u0220")
        buf.write("\u0222\3\2\2\2\u0221\u021f\3\2\2\2\u0222\u0223\7\24\2")
        buf.write("\2\u0223m\3\2\2\2\u0224\u0225\5j\66\2\u0225\u022b\5\22")
        buf.write("\n\2\u0226\u0227\5r:\2\u0227\u0228\5\22\n\2\u0228\u022a")
        buf.write("\3\2\2\2\u0229\u0226\3\2\2\2\u022a\u022d\3\2\2\2\u022b")
        buf.write("\u0229\3\2\2\2\u022b\u022c\3\2\2\2\u022c\u022e\3\2\2\2")
        buf.write("\u022d\u022b\3\2\2\2\u022e\u022f\5l\67\2\u022fo\3\2\2")
        buf.write("\2\u0230\u0237\5Z.\2\u0231\u0237\5b\62\2\u0232\u0237\5")
        buf.write("d\63\2\u0233\u0237\5f\64\2\u0234\u0237\5h\65\2\u0235\u0237")
        buf.write("\5n8\2\u0236\u0230\3\2\2\2\u0236\u0231\3\2\2\2\u0236\u0232")
        buf.write("\3\2\2\2\u0236\u0233\3\2\2\2\u0236\u0234\3\2\2\2\u0236")
        buf.write("\u0235\3\2\2\2\u0237q\3\2\2\2\u0238\u023a\7\25\2\2\u0239")
        buf.write("\u0238\3\2\2\2\u023a\u023d\3\2\2\2\u023b\u0239\3\2\2\2")
        buf.write("\u023b\u023c\3\2\2\2\u023c\u0249\3\2\2\2\u023d\u023b\3")
        buf.write("\2\2\2\u023e\u0242\5$\23\2\u023f\u0242\5X-\2\u0240\u0242")
        buf.write("\5p9\2\u0241\u023e\3\2\2\2\u0241\u023f\3\2\2\2\u0241\u0240")
        buf.write("\3\2\2\2\u0242\u0246\3\2\2\2\u0243\u0245\7\25\2\2\u0244")
        buf.write("\u0243\3\2\2\2\u0245\u0248\3\2\2\2\u0246\u0244\3\2\2\2")
        buf.write("\u0246\u0247\3\2\2\2\u0247\u024a\3\2\2\2\u0248\u0246\3")
        buf.write("\2\2\2\u0249\u0241\3\2\2\2\u0249\u024a\3\2\2\2\u024a\u024c")
        buf.write("\3\2\2\2\u024b\u024d\5\16\b\2\u024c\u024b\3\2\2\2\u024c")
        buf.write("\u024d\3\2\2\2\u024ds\3\2\2\2\u024e\u024f\5r:\2\u024f")
        buf.write("\u0250\5\22\n\2\u0250\u0252\3\2\2\2\u0251\u024e\3\2\2")
        buf.write("\2\u0252\u0255\3\2\2\2\u0253\u0251\3\2\2\2\u0253\u0254")
        buf.write("\3\2\2\2\u0254\u0256\3\2\2\2\u0255\u0253\3\2\2\2\u0256")
        buf.write("\u0257\5r:\2\u0257\u0258\7\2\2\3\u0258u\3\2\2\2Gy\u0080")
        buf.write("\u0088\u008e\u0098\u009f\u00a8\u00b1\u00b7\u00be\u00c5")
        buf.write("\u00ca\u00d1\u00d9\u00df\u00e6\u00ee\u00f4\u00fb\u0102")
        buf.write("\u0104\u010c\u0116\u011c\u011e\u0120\u0125\u0129\u0139")
        buf.write("\u013d\u0144\u014c\u0154\u0167\u016d\u0172\u017b\u0183")
        buf.write("\u018e\u0196\u019e\u01a2\u01a6\u01ab\u01af\u01b4\u01bd")
        buf.write("\u01c5\u01d1\u01d7\u01dc\u01e3\u01eb\u01ef\u01f4\u01f8")
        buf.write("\u020a\u0210\u0216\u021a\u021f\u022b\u0236\u023b\u0241")
        buf.write("\u0246\u0249\u024c\u0253")
        return buf.getvalue()


class SpudParser ( Parser ):

    grammarFileName = "Spud.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'.'", "'/'", "'#'", "'//'", "'/*'", "'*/'", 
                     "'+'", "'-'", "'$'", "'!'", "'@import'", "'@hold'", 
                     "'@holdunit'", "'@dec'", "'@frac'", "'@begin'", "'multiply'", 
                     "'@end'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "Inlinewhitespacetoken", 
                      "Returnwhitespacetoken", "Zerotoken", "Nonzerotoken", 
                      "Namesegment", "And", "Is", "Istype", "Issynonym", 
                      "Catchall" ]

    RULE_nonzerostring = 0
    RULE_decimallarge = 1
    RULE_decimalsmall = 2
    RULE_fractionsimple = 3
    RULE_fractioncomplex = 4
    RULE_number = 5
    RULE_onelinecomment = 6
    RULE_multilinecomment = 7
    RULE_linebreak = 8
    RULE_propspluselem = 9
    RULE_propsplus = 10
    RULE_propselem = 11
    RULE_props = 12
    RULE_propswildelem = 13
    RULE_propswild = 14
    RULE_basicname = 15
    RULE_basicnameplusprops = 16
    RULE_lineingredient = 17
    RULE_powname = 18
    RULE_pownamewithprops = 19
    RULE_basicnamewithpropswild = 20
    RULE_wildcardwithprops = 21
    RULE_wildcardwithpropswild = 22
    RULE_powwildwithprops = 23
    RULE_basicwildwithprops = 24
    RULE_ruleleftsimple = 25
    RULE_rulerightsimple = 26
    RULE_rulerightsimplewild = 27
    RULE_linerenaming = 28
    RULE_lineprefixing = 29
    RULE_lineinserting = 30
    RULE_singleconvertingleftelem = 31
    RULE_singleconvertingrightelem = 32
    RULE_singleconvertingleft = 33
    RULE_singleconvertingright = 34
    RULE_linesingleconverting = 35
    RULE_doubleconvertingleftsub1 = 36
    RULE_doubleconvertingleftsub2 = 37
    RULE_doubleconvertingleftelem = 38
    RULE_doubleconvertingrightelem = 39
    RULE_doubleconvertingleft = 40
    RULE_doubleconvertingright = 41
    RULE_linedoubleconverting = 42
    RULE_linestandardrule = 43
    RULE_atimport = 44
    RULE_atsub1 = 45
    RULE_atsub2 = 46
    RULE_atsub = 47
    RULE_athold = 48
    RULE_atholdunit = 49
    RULE_atdec = 50
    RULE_atfrac = 51
    RULE_atbegin = 52
    RULE_atend = 53
    RULE_atsection = 54
    RULE_lineat = 55
    RULE_line = 56
    RULE_spud = 57

    ruleNames =  [ "nonzerostring", "decimallarge", "decimalsmall", "fractionsimple", 
                   "fractioncomplex", "number", "onelinecomment", "multilinecomment", 
                   "linebreak", "propspluselem", "propsplus", "propselem", 
                   "props", "propswildelem", "propswild", "basicname", "basicnameplusprops", 
                   "lineingredient", "powname", "pownamewithprops", "basicnamewithpropswild", 
                   "wildcardwithprops", "wildcardwithpropswild", "powwildwithprops", 
                   "basicwildwithprops", "ruleleftsimple", "rulerightsimple", 
                   "rulerightsimplewild", "linerenaming", "lineprefixing", 
                   "lineinserting", "singleconvertingleftelem", "singleconvertingrightelem", 
                   "singleconvertingleft", "singleconvertingright", "linesingleconverting", 
                   "doubleconvertingleftsub1", "doubleconvertingleftsub2", 
                   "doubleconvertingleftelem", "doubleconvertingrightelem", 
                   "doubleconvertingleft", "doubleconvertingright", "linedoubleconverting", 
                   "linestandardrule", "atimport", "atsub1", "atsub2", "atsub", 
                   "athold", "atholdunit", "atdec", "atfrac", "atbegin", 
                   "atend", "atsection", "lineat", "line", "spud" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    Inlinewhitespacetoken=19
    Returnwhitespacetoken=20
    Zerotoken=21
    Nonzerotoken=22
    Namesegment=23
    And=24
    Is=25
    Istype=26
    Issynonym=27
    Catchall=28

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.9.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class NonzerostringContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Nonzerotoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Nonzerotoken)
            else:
                return self.getToken(SpudParser.Nonzerotoken, i)

        def Zerotoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Zerotoken)
            else:
                return self.getToken(SpudParser.Zerotoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_nonzerostring

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNonzerostring" ):
                listener.enterNonzerostring(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNonzerostring" ):
                listener.exitNonzerostring(self)




    def nonzerostring(self):

        localctx = SpudParser.NonzerostringContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_nonzerostring)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 119
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Zerotoken:
                self.state = 116
                self.match(SpudParser.Zerotoken)
                self.state = 121
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 122
            self.match(SpudParser.Nonzerotoken)
            self.state = 126
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Zerotoken or _la==SpudParser.Nonzerotoken:
                self.state = 123
                _la = self._input.LA(1)
                if not(_la==SpudParser.Zerotoken or _la==SpudParser.Nonzerotoken):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 128
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DecimallargeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def nonzerostring(self):
            return self.getTypedRuleContext(SpudParser.NonzerostringContext,0)


        def Zerotoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Zerotoken)
            else:
                return self.getToken(SpudParser.Zerotoken, i)

        def Nonzerotoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Nonzerotoken)
            else:
                return self.getToken(SpudParser.Nonzerotoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_decimallarge

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDecimallarge" ):
                listener.enterDecimallarge(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDecimallarge" ):
                listener.exitDecimallarge(self)




    def decimallarge(self):

        localctx = SpudParser.DecimallargeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_decimallarge)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 129
            self.nonzerostring()
            self.state = 130
            self.match(SpudParser.T__0)
            self.state = 134
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Zerotoken or _la==SpudParser.Nonzerotoken:
                self.state = 131
                _la = self._input.LA(1)
                if not(_la==SpudParser.Zerotoken or _la==SpudParser.Nonzerotoken):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 136
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DecimalsmallContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def nonzerostring(self):
            return self.getTypedRuleContext(SpudParser.NonzerostringContext,0)


        def Zerotoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Zerotoken)
            else:
                return self.getToken(SpudParser.Zerotoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_decimalsmall

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDecimalsmall" ):
                listener.enterDecimalsmall(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDecimalsmall" ):
                listener.exitDecimalsmall(self)




    def decimalsmall(self):

        localctx = SpudParser.DecimalsmallContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_decimalsmall)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 140
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Zerotoken:
                self.state = 137
                self.match(SpudParser.Zerotoken)
                self.state = 142
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 143
            self.match(SpudParser.T__0)
            self.state = 144
            self.nonzerostring()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FractionsimpleContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def nonzerostring(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.NonzerostringContext)
            else:
                return self.getTypedRuleContext(SpudParser.NonzerostringContext,i)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_fractionsimple

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFractionsimple" ):
                listener.enterFractionsimple(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFractionsimple" ):
                listener.exitFractionsimple(self)




    def fractionsimple(self):

        localctx = SpudParser.FractionsimpleContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_fractionsimple)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 146
            self.nonzerostring()
            self.state = 150
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 147
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 152
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 153
            self.match(SpudParser.T__1)
            self.state = 157
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 154
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 159
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 160
            self.nonzerostring()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FractioncomplexContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def nonzerostring(self):
            return self.getTypedRuleContext(SpudParser.NonzerostringContext,0)


        def fractionsimple(self):
            return self.getTypedRuleContext(SpudParser.FractionsimpleContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_fractioncomplex

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFractioncomplex" ):
                listener.enterFractioncomplex(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFractioncomplex" ):
                listener.exitFractioncomplex(self)




    def fractioncomplex(self):

        localctx = SpudParser.FractioncomplexContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_fractioncomplex)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 162
            self.nonzerostring()
            self.state = 164 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 163
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 166 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 168
            self.fractionsimple()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def nonzerostring(self):
            return self.getTypedRuleContext(SpudParser.NonzerostringContext,0)


        def decimallarge(self):
            return self.getTypedRuleContext(SpudParser.DecimallargeContext,0)


        def decimalsmall(self):
            return self.getTypedRuleContext(SpudParser.DecimalsmallContext,0)


        def fractionsimple(self):
            return self.getTypedRuleContext(SpudParser.FractionsimpleContext,0)


        def fractioncomplex(self):
            return self.getTypedRuleContext(SpudParser.FractioncomplexContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)




    def number(self):

        localctx = SpudParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_number)
        try:
            self.state = 175
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,7,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 170
                self.nonzerostring()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 171
                self.decimallarge()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 172
                self.decimalsmall()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 173
                self.fractionsimple()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 174
                self.fractioncomplex()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class OnelinecommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Returnwhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Returnwhitespacetoken)
            else:
                return self.getToken(SpudParser.Returnwhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_onelinecomment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOnelinecomment" ):
                listener.enterOnelinecomment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOnelinecomment" ):
                listener.exitOnelinecomment(self)




    def onelinecomment(self):

        localctx = SpudParser.OnelinecommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_onelinecomment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 177
            _la = self._input.LA(1)
            if not(_la==SpudParser.T__2 or _la==SpudParser.T__3):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 181
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,8,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 178
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==SpudParser.Returnwhitespacetoken:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume() 
                self.state = 183
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,8,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class MultilinecommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return SpudParser.RULE_multilinecomment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMultilinecomment" ):
                listener.enterMultilinecomment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMultilinecomment" ):
                listener.exitMultilinecomment(self)




    def multilinecomment(self):

        localctx = SpudParser.MultilinecommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_multilinecomment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 184
            self.match(SpudParser.T__4)
            self.state = 188
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << SpudParser.T__0) | (1 << SpudParser.T__1) | (1 << SpudParser.T__2) | (1 << SpudParser.T__3) | (1 << SpudParser.T__4) | (1 << SpudParser.T__6) | (1 << SpudParser.T__7) | (1 << SpudParser.T__8) | (1 << SpudParser.T__9) | (1 << SpudParser.T__10) | (1 << SpudParser.T__11) | (1 << SpudParser.T__12) | (1 << SpudParser.T__13) | (1 << SpudParser.T__14) | (1 << SpudParser.T__15) | (1 << SpudParser.T__16) | (1 << SpudParser.T__17) | (1 << SpudParser.Inlinewhitespacetoken) | (1 << SpudParser.Returnwhitespacetoken) | (1 << SpudParser.Zerotoken) | (1 << SpudParser.Nonzerotoken) | (1 << SpudParser.Namesegment) | (1 << SpudParser.And) | (1 << SpudParser.Is) | (1 << SpudParser.Istype) | (1 << SpudParser.Issynonym) | (1 << SpudParser.Catchall))) != 0):
                self.state = 185
                _la = self._input.LA(1)
                if _la <= 0 or _la==SpudParser.T__5:
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 190
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 191
            self.match(SpudParser.T__5)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LinebreakContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Returnwhitespacetoken(self):
            return self.getToken(SpudParser.Returnwhitespacetoken, 0)

        def multilinecomment(self):
            return self.getTypedRuleContext(SpudParser.MultilinecommentContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_linebreak

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLinebreak" ):
                listener.enterLinebreak(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLinebreak" ):
                listener.exitLinebreak(self)




    def linebreak(self):

        localctx = SpudParser.LinebreakContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_linebreak)
        try:
            self.state = 195
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.Returnwhitespacetoken]:
                self.enterOuterAlt(localctx, 1)
                self.state = 193
                self.match(SpudParser.Returnwhitespacetoken)
                pass
            elif token in [SpudParser.T__4]:
                self.enterOuterAlt(localctx, 2)
                self.state = 194
                self.multilinecomment()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropspluselemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Namesegment(self):
            return self.getToken(SpudParser.Namesegment, 0)

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_propspluselem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPropspluselem" ):
                listener.enterPropspluselem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPropspluselem" ):
                listener.exitPropspluselem(self)




    def propspluselem(self):

        localctx = SpudParser.PropspluselemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_propspluselem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 200
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 197
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 202
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 203
            self.match(SpudParser.T__6)
            self.state = 207
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 204
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 209
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 210
            self.match(SpudParser.Namesegment)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropsplusContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def propspluselem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.PropspluselemContext)
            else:
                return self.getTypedRuleContext(SpudParser.PropspluselemContext,i)


        def getRuleIndex(self):
            return SpudParser.RULE_propsplus

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPropsplus" ):
                listener.enterPropsplus(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPropsplus" ):
                listener.exitPropsplus(self)




    def propsplus(self):

        localctx = SpudParser.PropsplusContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_propsplus)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 215
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,13,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 212
                    self.propspluselem() 
                self.state = 217
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,13,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropselemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Namesegment(self):
            return self.getToken(SpudParser.Namesegment, 0)

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_propselem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPropselem" ):
                listener.enterPropselem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPropselem" ):
                listener.exitPropselem(self)




    def propselem(self):

        localctx = SpudParser.PropselemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_propselem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 221
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 218
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 223
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 224
            _la = self._input.LA(1)
            if not(_la==SpudParser.T__6 or _la==SpudParser.T__7):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 228
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 225
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 230
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 231
            self.match(SpudParser.Namesegment)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def propselem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.PropselemContext)
            else:
                return self.getTypedRuleContext(SpudParser.PropselemContext,i)


        def getRuleIndex(self):
            return SpudParser.RULE_props

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProps" ):
                listener.enterProps(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProps" ):
                listener.exitProps(self)




    def props(self):

        localctx = SpudParser.PropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_props)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 236
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,16,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 233
                    self.propselem() 
                self.state = 238
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,16,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropswildelemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_propswildelem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPropswildelem" ):
                listener.enterPropswildelem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPropswildelem" ):
                listener.exitPropswildelem(self)




    def propswildelem(self):

        localctx = SpudParser.PropswildelemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_propswildelem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 242
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 239
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 244
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 245
            self.match(SpudParser.T__7)
            self.state = 249
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 246
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 251
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 252
            self.match(SpudParser.T__8)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PropswildContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def propselem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.PropselemContext)
            else:
                return self.getTypedRuleContext(SpudParser.PropselemContext,i)


        def propswildelem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.PropswildelemContext)
            else:
                return self.getTypedRuleContext(SpudParser.PropswildelemContext,i)


        def getRuleIndex(self):
            return SpudParser.RULE_propswild

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPropswild" ):
                listener.enterPropswild(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPropswild" ):
                listener.exitPropswild(self)




    def propswild(self):

        localctx = SpudParser.PropswildContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_propswild)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 258
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,20,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 256
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,19,self._ctx)
                    if la_ == 1:
                        self.state = 254
                        self.propselem()
                        pass

                    elif la_ == 2:
                        self.state = 255
                        self.propswildelem()
                        pass

             
                self.state = 260
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,20,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BasicnameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Namesegment(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Namesegment)
            else:
                return self.getToken(SpudParser.Namesegment, i)

        def getRuleIndex(self):
            return SpudParser.RULE_basicname

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBasicname" ):
                listener.enterBasicname(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBasicname" ):
                listener.exitBasicname(self)




    def basicname(self):

        localctx = SpudParser.BasicnameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_basicname)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 261
            self.match(SpudParser.Namesegment)
            self.state = 266
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.T__0:
                self.state = 262
                self.match(SpudParser.T__0)
                self.state = 263
                self.match(SpudParser.Namesegment)
                self.state = 268
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BasicnamepluspropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicname(self):
            return self.getTypedRuleContext(SpudParser.BasicnameContext,0)


        def propsplus(self):
            return self.getTypedRuleContext(SpudParser.PropsplusContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_basicnameplusprops

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBasicnameplusprops" ):
                listener.enterBasicnameplusprops(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBasicnameplusprops" ):
                listener.exitBasicnameplusprops(self)




    def basicnameplusprops(self):

        localctx = SpudParser.BasicnamepluspropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_basicnameplusprops)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 269
            self.basicname()
            self.state = 270
            self.propsplus()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineingredientContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicnameplusprops(self):
            return self.getTypedRuleContext(SpudParser.BasicnamepluspropsContext,0)


        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def basicname(self):
            return self.getTypedRuleContext(SpudParser.BasicnameContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_lineingredient

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLineingredient" ):
                listener.enterLineingredient(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLineingredient" ):
                listener.exitLineingredient(self)




    def lineingredient(self):

        localctx = SpudParser.LineingredientContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_lineingredient)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 286
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << SpudParser.T__0) | (1 << SpudParser.Zerotoken) | (1 << SpudParser.Nonzerotoken))) != 0):
                self.state = 272
                self.number()
                self.state = 274 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 273
                    self.match(SpudParser.Inlinewhitespacetoken)
                    self.state = 276 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (_la==SpudParser.Inlinewhitespacetoken):
                        break

                self.state = 284
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,24,self._ctx)
                if la_ == 1:
                    self.state = 278
                    self.basicname()
                    self.state = 280 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while True:
                        self.state = 279
                        self.match(SpudParser.Inlinewhitespacetoken)
                        self.state = 282 
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        if not (_la==SpudParser.Inlinewhitespacetoken):
                            break





            self.state = 288
            self.basicnameplusprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PownameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicname(self):
            return self.getTypedRuleContext(SpudParser.BasicnameContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_powname

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPowname" ):
                listener.enterPowname(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPowname" ):
                listener.exitPowname(self)




    def powname(self):

        localctx = SpudParser.PownameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_powname)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 291
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==SpudParser.T__9:
                self.state = 290
                self.match(SpudParser.T__9)


            self.state = 293
            self.basicname()
            self.state = 295
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==SpudParser.T__9:
                self.state = 294
                self.match(SpudParser.T__9)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PownamewithpropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def powname(self):
            return self.getTypedRuleContext(SpudParser.PownameContext,0)


        def props(self):
            return self.getTypedRuleContext(SpudParser.PropsContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_pownamewithprops

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPownamewithprops" ):
                listener.enterPownamewithprops(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPownamewithprops" ):
                listener.exitPownamewithprops(self)




    def pownamewithprops(self):

        localctx = SpudParser.PownamewithpropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_pownamewithprops)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 297
            self.powname()
            self.state = 298
            self.props()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BasicnamewithpropswildContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicname(self):
            return self.getTypedRuleContext(SpudParser.BasicnameContext,0)


        def propswild(self):
            return self.getTypedRuleContext(SpudParser.PropswildContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_basicnamewithpropswild

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBasicnamewithpropswild" ):
                listener.enterBasicnamewithpropswild(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBasicnamewithpropswild" ):
                listener.exitBasicnamewithpropswild(self)




    def basicnamewithpropswild(self):

        localctx = SpudParser.BasicnamewithpropswildContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_basicnamewithpropswild)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 300
            self.basicname()
            self.state = 301
            self.propswild()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WildcardwithpropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def props(self):
            return self.getTypedRuleContext(SpudParser.PropsContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_wildcardwithprops

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWildcardwithprops" ):
                listener.enterWildcardwithprops(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWildcardwithprops" ):
                listener.exitWildcardwithprops(self)




    def wildcardwithprops(self):

        localctx = SpudParser.WildcardwithpropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_wildcardwithprops)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 303
            self.match(SpudParser.T__8)
            self.state = 304
            self.props()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WildcardwithpropswildContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def propswild(self):
            return self.getTypedRuleContext(SpudParser.PropswildContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_wildcardwithpropswild

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWildcardwithpropswild" ):
                listener.enterWildcardwithpropswild(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWildcardwithpropswild" ):
                listener.exitWildcardwithpropswild(self)




    def wildcardwithpropswild(self):

        localctx = SpudParser.WildcardwithpropswildContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_wildcardwithpropswild)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 306
            self.match(SpudParser.T__8)
            self.state = 307
            self.propswild()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PowwildwithpropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def pownamewithprops(self):
            return self.getTypedRuleContext(SpudParser.PownamewithpropsContext,0)


        def wildcardwithprops(self):
            return self.getTypedRuleContext(SpudParser.WildcardwithpropsContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_powwildwithprops

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPowwildwithprops" ):
                listener.enterPowwildwithprops(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPowwildwithprops" ):
                listener.exitPowwildwithprops(self)




    def powwildwithprops(self):

        localctx = SpudParser.PowwildwithpropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_powwildwithprops)
        try:
            self.state = 311
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__9, SpudParser.Namesegment]:
                self.enterOuterAlt(localctx, 1)
                self.state = 309
                self.pownamewithprops()
                pass
            elif token in [SpudParser.T__8]:
                self.enterOuterAlt(localctx, 2)
                self.state = 310
                self.wildcardwithprops()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BasicwildwithpropsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicnamewithpropswild(self):
            return self.getTypedRuleContext(SpudParser.BasicnamewithpropswildContext,0)


        def wildcardwithpropswild(self):
            return self.getTypedRuleContext(SpudParser.WildcardwithpropswildContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_basicwildwithprops

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBasicwildwithprops" ):
                listener.enterBasicwildwithprops(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBasicwildwithprops" ):
                listener.exitBasicwildwithprops(self)




    def basicwildwithprops(self):

        localctx = SpudParser.BasicwildwithpropsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_basicwildwithprops)
        try:
            self.state = 315
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.Namesegment]:
                self.enterOuterAlt(localctx, 1)
                self.state = 313
                self.basicnamewithpropswild()
                pass
            elif token in [SpudParser.T__8]:
                self.enterOuterAlt(localctx, 2)
                self.state = 314
                self.wildcardwithpropswild()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RuleleftsimpleContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def pownamewithprops(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.PownamewithpropsContext)
            else:
                return self.getTypedRuleContext(SpudParser.PownamewithpropsContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_ruleleftsimple

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRuleleftsimple" ):
                listener.enterRuleleftsimple(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRuleleftsimple" ):
                listener.exitRuleleftsimple(self)




    def ruleleftsimple(self):

        localctx = SpudParser.RuleleftsimpleContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_ruleleftsimple)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 317
            self.pownamewithprops()
            self.state = 322
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 318
                self.match(SpudParser.And)
                self.state = 319
                self.pownamewithprops()
                self.state = 324
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RulerightsimpleContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicnamewithpropswild(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.BasicnamewithpropswildContext)
            else:
                return self.getTypedRuleContext(SpudParser.BasicnamewithpropswildContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_rulerightsimple

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRulerightsimple" ):
                listener.enterRulerightsimple(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRulerightsimple" ):
                listener.exitRulerightsimple(self)




    def rulerightsimple(self):

        localctx = SpudParser.RulerightsimpleContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_rulerightsimple)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 325
            self.basicnamewithpropswild()
            self.state = 330
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 326
                self.match(SpudParser.And)
                self.state = 327
                self.basicnamewithpropswild()
                self.state = 332
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RulerightsimplewildContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicwildwithprops(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.BasicwildwithpropsContext)
            else:
                return self.getTypedRuleContext(SpudParser.BasicwildwithpropsContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_rulerightsimplewild

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRulerightsimplewild" ):
                listener.enterRulerightsimplewild(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRulerightsimplewild" ):
                listener.exitRulerightsimplewild(self)




    def rulerightsimplewild(self):

        localctx = SpudParser.RulerightsimplewildContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_rulerightsimplewild)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 333
            self.basicwildwithprops()
            self.state = 338
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 334
                self.match(SpudParser.And)
                self.state = 335
                self.basicwildwithprops()
                self.state = 340
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LinerenamingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ruleleftsimple(self):
            return self.getTypedRuleContext(SpudParser.RuleleftsimpleContext,0)


        def Is(self):
            return self.getToken(SpudParser.Is, 0)

        def rulerightsimplewild(self):
            return self.getTypedRuleContext(SpudParser.RulerightsimplewildContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_linerenaming

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLinerenaming" ):
                listener.enterLinerenaming(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLinerenaming" ):
                listener.exitLinerenaming(self)




    def linerenaming(self):

        localctx = SpudParser.LinerenamingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_linerenaming)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 341
            self.ruleleftsimple()
            self.state = 342
            self.match(SpudParser.Is)
            self.state = 343
            self.rulerightsimplewild()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineprefixingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ruleleftsimple(self):
            return self.getTypedRuleContext(SpudParser.RuleleftsimpleContext,0)


        def Istype(self):
            return self.getToken(SpudParser.Istype, 0)

        def rulerightsimplewild(self):
            return self.getTypedRuleContext(SpudParser.RulerightsimplewildContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_lineprefixing

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLineprefixing" ):
                listener.enterLineprefixing(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLineprefixing" ):
                listener.exitLineprefixing(self)




    def lineprefixing(self):

        localctx = SpudParser.LineprefixingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_lineprefixing)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 345
            self.ruleleftsimple()
            self.state = 346
            self.match(SpudParser.Istype)
            self.state = 347
            self.rulerightsimplewild()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineinsertingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ruleleftsimple(self):
            return self.getTypedRuleContext(SpudParser.RuleleftsimpleContext,0)


        def Issynonym(self):
            return self.getToken(SpudParser.Issynonym, 0)

        def rulerightsimple(self):
            return self.getTypedRuleContext(SpudParser.RulerightsimpleContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_lineinserting

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLineinserting" ):
                listener.enterLineinserting(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLineinserting" ):
                listener.exitLineinserting(self)




    def lineinserting(self):

        localctx = SpudParser.LineinsertingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_lineinserting)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 349
            self.ruleleftsimple()
            self.state = 350
            self.match(SpudParser.Issynonym)
            self.state = 351
            self.rulerightsimple()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SingleconvertingleftelemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def pownamewithprops(self):
            return self.getTypedRuleContext(SpudParser.PownamewithpropsContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_singleconvertingleftelem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleconvertingleftelem" ):
                listener.enterSingleconvertingleftelem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleconvertingleftelem" ):
                listener.exitSingleconvertingleftelem(self)




    def singleconvertingleftelem(self):

        localctx = SpudParser.SingleconvertingleftelemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_singleconvertingleftelem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 353
            self.number()
            self.state = 355 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 354
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 357 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 359
            self.pownamewithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SingleconvertingrightelemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicwildwithprops(self):
            return self.getTypedRuleContext(SpudParser.BasicwildwithpropsContext,0)


        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_singleconvertingrightelem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleconvertingrightelem" ):
                listener.enterSingleconvertingrightelem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleconvertingrightelem" ):
                listener.exitSingleconvertingrightelem(self)




    def singleconvertingrightelem(self):

        localctx = SpudParser.SingleconvertingrightelemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_singleconvertingrightelem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 363
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__0, SpudParser.Zerotoken, SpudParser.Nonzerotoken]:
                self.state = 361
                self.number()
                pass
            elif token in [SpudParser.T__8]:
                self.state = 362
                self.match(SpudParser.T__8)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 366 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 365
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 368 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 370
            self.basicwildwithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SingleconvertingleftContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def singleconvertingleftelem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.SingleconvertingleftelemContext)
            else:
                return self.getTypedRuleContext(SpudParser.SingleconvertingleftelemContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_singleconvertingleft

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleconvertingleft" ):
                listener.enterSingleconvertingleft(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleconvertingleft" ):
                listener.exitSingleconvertingleft(self)




    def singleconvertingleft(self):

        localctx = SpudParser.SingleconvertingleftContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_singleconvertingleft)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 372
            self.singleconvertingleftelem()
            self.state = 377
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 373
                self.match(SpudParser.And)
                self.state = 374
                self.singleconvertingleftelem()
                self.state = 379
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SingleconvertingrightContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def singleconvertingrightelem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.SingleconvertingrightelemContext)
            else:
                return self.getTypedRuleContext(SpudParser.SingleconvertingrightelemContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_singleconvertingright

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleconvertingright" ):
                listener.enterSingleconvertingright(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleconvertingright" ):
                listener.exitSingleconvertingright(self)




    def singleconvertingright(self):

        localctx = SpudParser.SingleconvertingrightContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_singleconvertingright)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 380
            self.singleconvertingrightelem()
            self.state = 385
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 381
                self.match(SpudParser.And)
                self.state = 382
                self.singleconvertingrightelem()
                self.state = 387
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LinesingleconvertingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def singleconvertingleft(self):
            return self.getTypedRuleContext(SpudParser.SingleconvertingleftContext,0)


        def Is(self):
            return self.getToken(SpudParser.Is, 0)

        def singleconvertingright(self):
            return self.getTypedRuleContext(SpudParser.SingleconvertingrightContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_linesingleconverting

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLinesingleconverting" ):
                listener.enterLinesingleconverting(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLinesingleconverting" ):
                listener.exitLinesingleconverting(self)




    def linesingleconverting(self):

        localctx = SpudParser.LinesingleconvertingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_linesingleconverting)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 388
            self.singleconvertingleft()
            self.state = 389
            self.match(SpudParser.Is)
            self.state = 390
            self.singleconvertingright()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Doubleconvertingleftsub1Context(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def pownamewithprops(self):
            return self.getTypedRuleContext(SpudParser.PownamewithpropsContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingleftsub1

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingleftsub1" ):
                listener.enterDoubleconvertingleftsub1(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingleftsub1" ):
                listener.exitDoubleconvertingleftsub1(self)




    def doubleconvertingleftsub1(self):

        localctx = SpudParser.Doubleconvertingleftsub1Context(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_doubleconvertingleftsub1)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 392
            self.match(SpudParser.T__8)
            self.state = 394 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 393
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 396 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 398
            self.pownamewithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Doubleconvertingleftsub2Context(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def powname(self):
            return self.getTypedRuleContext(SpudParser.PownameContext,0)


        def powwildwithprops(self):
            return self.getTypedRuleContext(SpudParser.PowwildwithpropsContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingleftsub2

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingleftsub2" ):
                listener.enterDoubleconvertingleftsub2(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingleftsub2" ):
                listener.exitDoubleconvertingleftsub2(self)




    def doubleconvertingleftsub2(self):

        localctx = SpudParser.Doubleconvertingleftsub2Context(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_doubleconvertingleftsub2)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 400
            self.powname()
            self.state = 402 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 401
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 404 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 406
            self.powwildwithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DoubleconvertingleftelemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def doubleconvertingleftsub1(self):
            return self.getTypedRuleContext(SpudParser.Doubleconvertingleftsub1Context,0)


        def doubleconvertingleftsub2(self):
            return self.getTypedRuleContext(SpudParser.Doubleconvertingleftsub2Context,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingleftelem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingleftelem" ):
                listener.enterDoubleconvertingleftelem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingleftelem" ):
                listener.exitDoubleconvertingleftelem(self)




    def doubleconvertingleftelem(self):

        localctx = SpudParser.DoubleconvertingleftelemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_doubleconvertingleftelem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 408
            self.number()
            self.state = 410 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 409
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 412 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 416
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__8]:
                self.state = 414
                self.doubleconvertingleftsub1()
                pass
            elif token in [SpudParser.T__9, SpudParser.Namesegment]:
                self.state = 415
                self.doubleconvertingleftsub2()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DoubleconvertingrightelemContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def basicwildwithprops(self):
            return self.getTypedRuleContext(SpudParser.BasicwildwithpropsContext,0)


        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def basicname(self):
            return self.getTypedRuleContext(SpudParser.BasicnameContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingrightelem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingrightelem" ):
                listener.enterDoubleconvertingrightelem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingrightelem" ):
                listener.exitDoubleconvertingrightelem(self)




    def doubleconvertingrightelem(self):

        localctx = SpudParser.DoubleconvertingrightelemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_doubleconvertingrightelem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 420
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__0, SpudParser.Zerotoken, SpudParser.Nonzerotoken]:
                self.state = 418
                self.number()
                pass
            elif token in [SpudParser.T__8]:
                self.state = 419
                self.match(SpudParser.T__8)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 423 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 422
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 425 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 429
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.Namesegment]:
                self.state = 427
                self.basicname()
                pass
            elif token in [SpudParser.T__8]:
                self.state = 428
                self.match(SpudParser.T__8)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 432 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 431
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 434 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 436
            self.basicwildwithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DoubleconvertingleftContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def doubleconvertingleftelem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.DoubleconvertingleftelemContext)
            else:
                return self.getTypedRuleContext(SpudParser.DoubleconvertingleftelemContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingleft

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingleft" ):
                listener.enterDoubleconvertingleft(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingleft" ):
                listener.exitDoubleconvertingleft(self)




    def doubleconvertingleft(self):

        localctx = SpudParser.DoubleconvertingleftContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_doubleconvertingleft)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 438
            self.doubleconvertingleftelem()
            self.state = 443
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 439
                self.match(SpudParser.And)
                self.state = 440
                self.doubleconvertingleftelem()
                self.state = 445
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DoubleconvertingrightContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def doubleconvertingrightelem(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.DoubleconvertingrightelemContext)
            else:
                return self.getTypedRuleContext(SpudParser.DoubleconvertingrightelemContext,i)


        def And(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.And)
            else:
                return self.getToken(SpudParser.And, i)

        def getRuleIndex(self):
            return SpudParser.RULE_doubleconvertingright

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDoubleconvertingright" ):
                listener.enterDoubleconvertingright(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDoubleconvertingright" ):
                listener.exitDoubleconvertingright(self)




    def doubleconvertingright(self):

        localctx = SpudParser.DoubleconvertingrightContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_doubleconvertingright)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 446
            self.doubleconvertingrightelem()
            self.state = 451
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.And:
                self.state = 447
                self.match(SpudParser.And)
                self.state = 448
                self.doubleconvertingrightelem()
                self.state = 453
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LinedoubleconvertingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def doubleconvertingleft(self):
            return self.getTypedRuleContext(SpudParser.DoubleconvertingleftContext,0)


        def Is(self):
            return self.getToken(SpudParser.Is, 0)

        def doubleconvertingright(self):
            return self.getTypedRuleContext(SpudParser.DoubleconvertingrightContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_linedoubleconverting

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLinedoubleconverting" ):
                listener.enterLinedoubleconverting(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLinedoubleconverting" ):
                listener.exitLinedoubleconverting(self)




    def linedoubleconverting(self):

        localctx = SpudParser.LinedoubleconvertingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_linedoubleconverting)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 454
            self.doubleconvertingleft()
            self.state = 455
            self.match(SpudParser.Is)
            self.state = 456
            self.doubleconvertingright()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LinestandardruleContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def linerenaming(self):
            return self.getTypedRuleContext(SpudParser.LinerenamingContext,0)


        def lineprefixing(self):
            return self.getTypedRuleContext(SpudParser.LineprefixingContext,0)


        def lineinserting(self):
            return self.getTypedRuleContext(SpudParser.LineinsertingContext,0)


        def linesingleconverting(self):
            return self.getTypedRuleContext(SpudParser.LinesingleconvertingContext,0)


        def linedoubleconverting(self):
            return self.getTypedRuleContext(SpudParser.LinedoubleconvertingContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_linestandardrule

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLinestandardrule" ):
                listener.enterLinestandardrule(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLinestandardrule" ):
                listener.exitLinestandardrule(self)




    def linestandardrule(self):

        localctx = SpudParser.LinestandardruleContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_linestandardrule)
        try:
            self.state = 463
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,48,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 458
                self.linerenaming()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 459
                self.lineprefixing()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 460
                self.lineinserting()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 461
                self.linesingleconverting()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 462
                self.linedoubleconverting()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtimportContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def Returnwhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Returnwhitespacetoken)
            else:
                return self.getToken(SpudParser.Returnwhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_atimport

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtimport" ):
                listener.enterAtimport(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtimport" ):
                listener.exitAtimport(self)




    def atimport(self):

        localctx = SpudParser.AtimportContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_atimport)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 465
            self.match(SpudParser.T__10)
            self.state = 467 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 466
                    self.match(SpudParser.Inlinewhitespacetoken)

                else:
                    raise NoViableAltException(self)
                self.state = 469 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,49,self._ctx)

            self.state = 474
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,50,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 471
                    _la = self._input.LA(1)
                    if _la <= 0 or _la==SpudParser.Returnwhitespacetoken:
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume() 
                self.state = 476
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,50,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Atsub1Context(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def pownamewithprops(self):
            return self.getTypedRuleContext(SpudParser.PownamewithpropsContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_atsub1

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtsub1" ):
                listener.enterAtsub1(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtsub1" ):
                listener.exitAtsub1(self)




    def atsub1(self):

        localctx = SpudParser.Atsub1Context(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_atsub1)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 477
            self.match(SpudParser.T__8)
            self.state = 479 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 478
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 481 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 483
            self.pownamewithprops()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Atsub2Context(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def powname(self):
            return self.getTypedRuleContext(SpudParser.PownameContext,0)


        def powwildwithprops(self):
            return self.getTypedRuleContext(SpudParser.PowwildwithpropsContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_atsub2

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtsub2" ):
                listener.enterAtsub2(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtsub2" ):
                listener.exitAtsub2(self)




    def atsub2(self):

        localctx = SpudParser.Atsub2Context(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_atsub2)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 485
            self.powname()
            self.state = 487 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 486
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 489 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 493
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,53,self._ctx)
            if la_ == 1:
                self.state = 491
                self.powwildwithprops()
                pass

            elif la_ == 2:
                self.state = 492
                self.match(SpudParser.T__8)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtsubContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atsub1(self):
            return self.getTypedRuleContext(SpudParser.Atsub1Context,0)


        def atsub2(self):
            return self.getTypedRuleContext(SpudParser.Atsub2Context,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_atsub

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtsub" ):
                listener.enterAtsub(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtsub" ):
                listener.exitAtsub(self)




    def atsub(self):

        localctx = SpudParser.AtsubContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_atsub)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 496 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 495
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 498 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 502
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__8]:
                self.state = 500
                self.atsub1()
                pass
            elif token in [SpudParser.T__9, SpudParser.Namesegment]:
                self.state = 501
                self.atsub2()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtholdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atsub(self):
            return self.getTypedRuleContext(SpudParser.AtsubContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_athold

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAthold" ):
                listener.enterAthold(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAthold" ):
                listener.exitAthold(self)




    def athold(self):

        localctx = SpudParser.AtholdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_athold)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 504
            self.match(SpudParser.T__11)
            self.state = 505
            self.atsub()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtholdunitContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atsub(self):
            return self.getTypedRuleContext(SpudParser.AtsubContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_atholdunit

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtholdunit" ):
                listener.enterAtholdunit(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtholdunit" ):
                listener.exitAtholdunit(self)




    def atholdunit(self):

        localctx = SpudParser.AtholdunitContext(self, self._ctx, self.state)
        self.enterRule(localctx, 98, self.RULE_atholdunit)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 507
            self.match(SpudParser.T__12)
            self.state = 508
            self.atsub()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtdecContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atsub(self):
            return self.getTypedRuleContext(SpudParser.AtsubContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_atdec

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtdec" ):
                listener.enterAtdec(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtdec" ):
                listener.exitAtdec(self)




    def atdec(self):

        localctx = SpudParser.AtdecContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_atdec)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 510
            self.match(SpudParser.T__13)
            self.state = 511
            self.atsub()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtfracContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atsub(self):
            return self.getTypedRuleContext(SpudParser.AtsubContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_atfrac

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtfrac" ):
                listener.enterAtfrac(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtfrac" ):
                listener.exitAtfrac(self)




    def atfrac(self):

        localctx = SpudParser.AtfracContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_atfrac)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 513
            self.match(SpudParser.T__14)
            self.state = 514
            self.atsub()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtbeginContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def number(self):
            return self.getTypedRuleContext(SpudParser.NumberContext,0)


        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def onelinecomment(self):
            return self.getTypedRuleContext(SpudParser.OnelinecommentContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_atbegin

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtbegin" ):
                listener.enterAtbegin(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtbegin" ):
                listener.exitAtbegin(self)




    def atbegin(self):

        localctx = SpudParser.AtbeginContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_atbegin)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 516
            self.match(SpudParser.T__15)
            self.state = 518 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 517
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 520 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 522
            self.match(SpudParser.T__16)
            self.state = 524 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 523
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 526 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==SpudParser.Inlinewhitespacetoken):
                    break

            self.state = 528
            self.number()
            self.state = 532
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 529
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 534
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 536
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==SpudParser.T__2 or _la==SpudParser.T__3:
                self.state = 535
                self.onelinecomment()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtendContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def getRuleIndex(self):
            return SpudParser.RULE_atend

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtend" ):
                listener.enterAtend(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtend" ):
                listener.exitAtend(self)




    def atend(self):

        localctx = SpudParser.AtendContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_atend)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 541
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 538
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 543
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 544
            self.match(SpudParser.T__17)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtsectionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atbegin(self):
            return self.getTypedRuleContext(SpudParser.AtbeginContext,0)


        def linebreak(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.LinebreakContext)
            else:
                return self.getTypedRuleContext(SpudParser.LinebreakContext,i)


        def atend(self):
            return self.getTypedRuleContext(SpudParser.AtendContext,0)


        def line(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.LineContext)
            else:
                return self.getTypedRuleContext(SpudParser.LineContext,i)


        def getRuleIndex(self):
            return SpudParser.RULE_atsection

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtsection" ):
                listener.enterAtsection(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtsection" ):
                listener.exitAtsection(self)




    def atsection(self):

        localctx = SpudParser.AtsectionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 108, self.RULE_atsection)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 546
            self.atbegin()
            self.state = 547
            self.linebreak()
            self.state = 553
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,61,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 548
                    self.line()
                    self.state = 549
                    self.linebreak() 
                self.state = 555
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,61,self._ctx)

            self.state = 556
            self.atend()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineatContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atimport(self):
            return self.getTypedRuleContext(SpudParser.AtimportContext,0)


        def athold(self):
            return self.getTypedRuleContext(SpudParser.AtholdContext,0)


        def atholdunit(self):
            return self.getTypedRuleContext(SpudParser.AtholdunitContext,0)


        def atdec(self):
            return self.getTypedRuleContext(SpudParser.AtdecContext,0)


        def atfrac(self):
            return self.getTypedRuleContext(SpudParser.AtfracContext,0)


        def atsection(self):
            return self.getTypedRuleContext(SpudParser.AtsectionContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_lineat

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLineat" ):
                listener.enterLineat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLineat" ):
                listener.exitLineat(self)




    def lineat(self):

        localctx = SpudParser.LineatContext(self, self._ctx, self.state)
        self.enterRule(localctx, 110, self.RULE_lineat)
        try:
            self.state = 564
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [SpudParser.T__10]:
                self.enterOuterAlt(localctx, 1)
                self.state = 558
                self.atimport()
                pass
            elif token in [SpudParser.T__11]:
                self.enterOuterAlt(localctx, 2)
                self.state = 559
                self.athold()
                pass
            elif token in [SpudParser.T__12]:
                self.enterOuterAlt(localctx, 3)
                self.state = 560
                self.atholdunit()
                pass
            elif token in [SpudParser.T__13]:
                self.enterOuterAlt(localctx, 4)
                self.state = 561
                self.atdec()
                pass
            elif token in [SpudParser.T__14]:
                self.enterOuterAlt(localctx, 5)
                self.state = 562
                self.atfrac()
                pass
            elif token in [SpudParser.T__15]:
                self.enterOuterAlt(localctx, 6)
                self.state = 563
                self.atsection()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LineContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Inlinewhitespacetoken(self, i:int=None):
            if i is None:
                return self.getTokens(SpudParser.Inlinewhitespacetoken)
            else:
                return self.getToken(SpudParser.Inlinewhitespacetoken, i)

        def onelinecomment(self):
            return self.getTypedRuleContext(SpudParser.OnelinecommentContext,0)


        def lineingredient(self):
            return self.getTypedRuleContext(SpudParser.LineingredientContext,0)


        def linestandardrule(self):
            return self.getTypedRuleContext(SpudParser.LinestandardruleContext,0)


        def lineat(self):
            return self.getTypedRuleContext(SpudParser.LineatContext,0)


        def getRuleIndex(self):
            return SpudParser.RULE_line

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLine" ):
                listener.enterLine(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLine" ):
                listener.exitLine(self)




    def line(self):

        localctx = SpudParser.LineContext(self, self._ctx, self.state)
        self.enterRule(localctx, 112, self.RULE_line)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 569
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==SpudParser.Inlinewhitespacetoken:
                self.state = 566
                self.match(SpudParser.Inlinewhitespacetoken)
                self.state = 571
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 583
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << SpudParser.T__0) | (1 << SpudParser.T__9) | (1 << SpudParser.T__10) | (1 << SpudParser.T__11) | (1 << SpudParser.T__12) | (1 << SpudParser.T__13) | (1 << SpudParser.T__14) | (1 << SpudParser.T__15) | (1 << SpudParser.Zerotoken) | (1 << SpudParser.Nonzerotoken) | (1 << SpudParser.Namesegment))) != 0):
                self.state = 575
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,64,self._ctx)
                if la_ == 1:
                    self.state = 572
                    self.lineingredient()
                    pass

                elif la_ == 2:
                    self.state = 573
                    self.linestandardrule()
                    pass

                elif la_ == 3:
                    self.state = 574
                    self.lineat()
                    pass


                self.state = 580
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==SpudParser.Inlinewhitespacetoken:
                    self.state = 577
                    self.match(SpudParser.Inlinewhitespacetoken)
                    self.state = 582
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 586
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==SpudParser.T__2 or _la==SpudParser.T__3:
                self.state = 585
                self.onelinecomment()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SpudContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def line(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.LineContext)
            else:
                return self.getTypedRuleContext(SpudParser.LineContext,i)


        def EOF(self):
            return self.getToken(SpudParser.EOF, 0)

        def linebreak(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SpudParser.LinebreakContext)
            else:
                return self.getTypedRuleContext(SpudParser.LinebreakContext,i)


        def getRuleIndex(self):
            return SpudParser.RULE_spud

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpud" ):
                listener.enterSpud(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpud" ):
                listener.exitSpud(self)




    def spud(self):

        localctx = SpudParser.SpudContext(self, self._ctx, self.state)
        self.enterRule(localctx, 114, self.RULE_spud)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 593
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,68,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 588
                    self.line()
                    self.state = 589
                    self.linebreak() 
                self.state = 595
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,68,self._ctx)

            self.state = 596
            self.line()
            self.state = 597
            self.match(SpudParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx





